from dotenv import load_dotenv
from genai import Credentials, Client
from genai.schema import TextGenerationParameters, TextGenerationReturnOptions
import jsonlines
import random
import json
from tqdm import tqdm
import os

random.seed(42)

def get_bam_response(prompt, model_id):
    load_dotenv()
    credentials = Credentials.from_env()
    stop = ["\nINPUT"]
    if "granite" in model_id:
        stop = ["\nINPUT","<|endoftext|>"]
    parameters = TextGenerationParameters(decoding_method="greedy", max_new_tokens=600, temperature=0.05, stop_sequences=stop, include_stop_sequence=False)
    client = Client(credentials=credentials)
    responses = list(
        client.text.generation.create(
            model_id=model_id,
            inputs=[prompt],
            parameters=parameters,
        )
    )
    return responses[0].results[0].generated_text

all_examples = []
spotify_all_examples = []
tmdb_all_examples = []
with jsonlines.open('jsonlines_data/train.jsonl') as reader:
    for i, obj in enumerate(reader):
        # if i == 25:
        #     print(len(spotify_all_examples))
        all_examples.append(obj)
        if obj["domain"] == "spotify":
            spotify_all_examples.append(obj)
        if obj["domain"] == "tmdb":
            tmdb_all_examples.append(obj)

spotify_examples = [spotify_all_examples[0], spotify_all_examples[16], spotify_all_examples[25]]
tmdb_examples = [tmdb_all_examples[0], tmdb_all_examples[16], tmdb_all_examples[25]]


spotify_prompt_template = f"""
You are a python code expert. Generate python code as per the user utterance as shown below:

INPUT:

{spotify_examples[0]["input"]}

THOUGHT:

- From the user utterance, I understand I need to call the following sequence of functions:
    - GET_me_player_currently_playing
    - GET_tracks__id_
    - PUT_me_following
- If response_obj is the response of GET_me_player_currently_playing, looking at the GraphQL schema of GET_me_player_currently_playing I must use response_obj.item.id as the `id` parameter of GET_tracks__id_
- If response_obj1 is the response of GET_tracks__id_, looking at the GraphQL schema of GET_tracks__id_ I must use response_obj1.artists[0].id as the `id` parameter of PUT_me_following.
- The `type` (string) parameter of PUT_me_following must be "artist" as specified by the function signature of PUT_me_following

OUTPUT:

{spotify_examples[0]["output"]}

INPUT:

{spotify_examples[1]["input"]}

THOUGHT:

- From the user utterance, I understand I need to call the following sequence of functions:
    - GET_me_following
    - GET_artists__id__top_tracks
    - PUT_me_tracks
- The parameter `type` (string) of GET_me_following must be "artist" from the user utterance
- If response_obj is the response of GET_me_following, looking at the GraphQL schema of GET_me_following I must use response_obj.artists.items[0].id as the `id` (integer) parameter of GET_artists__id__top_tracks
- If response_obj1 is the response of GET_artists__id__top_tracks, since the type of the `ids` parameter of PUT_me_tracks is string, looking at the GraphQL schema of GET_artists__id__top_tracks I must use the python code ",".join([track.id for track in response_obj1.tracks]) as the `ids` (string) parameter of PUT_me_tracks

OUTPUT:

{spotify_examples[1]["output"]}

INPUT:

<<input>>

THOUGHT:

"""

tmdb_prompt_template = f"""
You are a python code expert. Generate python code as per the user utterance as shown below:

INPUT:

{tmdb_examples[0]["input"]}

THOUGHT:

- From the user utterance, I understand I need to call the following sequence of functions:
    - GET_search_movie
    - GET_search_movie
- The parameter `query` (string) of the first call to GET_search_movie must be "Avatar" from the user utterance
- The parameter `query` (string) of the second call to GET_search_movie must be "Avatar: The Way of Water" from the user utterance

OUTPUT:

{tmdb_examples[0]["output"]}

INPUT:

{tmdb_examples[1]["input"]}

THOUGHT:

- From the user utterance, I understand I need to call the following sequence of functions:
    - GET_search_movie
    - GET_movie__movie_id__credits
    - GET_person__person_id__movie_credits
- The parameter `query` (string) of GET_search_movie must be "Django Unchained" from the user utterance
- If response_obj is the response of GET_search_movie, looking at the GraphQL schema of GET_search_movie I must use response_obj.results[0].id as the `id` (integer) parameter of GET_movie__movie_id__credits
- If response_obj1 is the response of GET_movie__movie_id__credits, since the type of the `person_id` parameter of PUT_me_tracks is integer, looking at the GraphQL schema of GET_movie__movie_id__credits I must use the python code [person.id for person in response_obj1.crew if person.job.lower()=="director"][0] as the `person_id` (integer) parameter of GET_person__person_id__movie_credits

OUTPUT:

{tmdb_examples[1]["output"]}

INPUT:

<<input>>

THOUGHT:

"""

# print(spotify_examples[0]["utterance"])
# print(json.dumps(eval(spotify_examples[0]["function_specs"]), indent=4))
# print(spotify_examples[0]["output"])

# print(spotify_examples[1]["utterance"])
# print(json.dumps(eval(spotify_examples[1]["function_specs"]), indent=4))
# print(spotify_examples[1]["output"])

# print(spotify_examples[2]["utterance"])
# print(json.dumps(eval(spotify_examples[2]["function_specs"]), indent=4))
# print(spotify_examples[2]["output"])

# print(tmdb_examples[0]["utterance"])
# print(json.dumps(eval(tmdb_examples[0]["function_specs"]), indent=4))
# print(tmdb_examples[0]["output"])

# print(tmdb_examples[1]["utterance"])
# print(json.dumps(eval(tmdb_examples[1]["function_specs"]), indent=4))
# print(tmdb_examples[1]["output"])

# print(tmdb_examples[2]["utterance"])
# print(json.dumps(eval(tmdb_examples[2]["function_specs"]), indent=4))
# print(tmdb_examples[2]["output"])

# quit()
test_samples = []
with jsonlines.open('jsonlines_data/test.jsonl') as reader:
    for obj in reader:
        test_samples.append(obj)

spotify_test_samples = []
with jsonlines.open('jsonlines_data/test_spotify.jsonl') as reader:
    for obj in reader:
        spotify_test_samples.append(obj)

tmdb_test_samples = []
with jsonlines.open('jsonlines_data/test_tmdb.jsonl') as reader:
    for obj in reader:
        tmdb_test_samples.append(obj)


model_id = "ibm/granite-34b-code-instruct"
output_folder = "few_shot_prompting/granite_output/"

with open(os.path.join(output_folder, "output_spotify.jsonl"), "w") as out:
    for sample in tqdm(spotify_test_samples):
        prompt = spotify_prompt_template.replace("<<input>>", sample["input"])
        response = get_bam_response(prompt, model_id)
        code = response.split("OUTPUT:", 1)[1].strip()
        d = {"input": sample["input"], "expected_output": sample["output"], "generated_output": code}
        print(json.dumps(d), file=out)

with open(os.path.join(output_folder, "output_tmdb.jsonl"), "w") as out:
    for sample in tqdm(tmdb_test_samples):
        prompt = tmdb_prompt_template.replace("<<input>>", sample["input"])
        response = get_bam_response(prompt, model_id)
        code = response.split("OUTPUT:", 1)[1].strip()
        d = {"input": sample["input"], "expected_output": sample["output"], "generated_output": code}
        print(json.dumps(d), file=out)

with open(os.path.join(output_folder, "output.jsonl"), "w") as out:
    for sample in tqdm(test_samples):
        if sample["domain"] == "spotify":
            prompt = spotify_prompt_template.replace("<<input>>", sample["input"])
        else:
            prompt = tmdb_prompt_template.replace("<<input>>", sample["input"])
        response = get_bam_response(prompt, model_id)
        code = response.split("OUTPUT:", 1)[1].strip()
        d = {"input": sample["input"], "expected_output": sample["output"], "generated_output": code}
        print(json.dumps(d), file=out)